<?php namespace App\Commands;

use CodeIgniter\CLI\BaseCommand;
use CodeIgniter\CLI\CLI;

class MakeModel extends BaseCommand
{
	protected $group = 'ErtikazOS';
	protected $name = 'make:model';
	protected $description = 'Create a new model class.';
	protected $usage = 'make:model [namespace] [model] [fields] or [options]';
	protected $arguments = [
		'namespace' => 'The namespace of your application.',
		'model' => 'The name of the model class.',
		'fields' => 'The list of fields separated by comma.',
	];
	protected $options = [
		'-t' => 'Set data table name',
	];

	public function run(array $params = [])
	{
		if (!isset($params[0]) || empty($params[0]))
		{
			CLI::error('"namespace" argument is missing.');
			CLI::write('');
			return;
		}
		if (!isset($params[1]) || empty($params[1]))
		{
			CLI::error('"model" argument is missing.');
			CLI::write('');
			return;
		}

		// Class names must start with an uppercase letter.
		$namespace  = ucfirst(array_shift($params));
		$classname  = ucfirst(array_shift($params));
		$tablename  = strtolower($classname).'s';
		$table      = CLI::getOption('t');

		// if table set but empty , show error
		if ($table == 1) 
		{
			CLI::error('"table" argument is missing.');
			CLI::write('');
			return;
		}
		// else if second argument in not set , show error
		elseif (empty($params[0]) && empty($table))
		{
			CLI::error('"fields" argument is missing.');
			CLI::write('');
			return;
		}

		// if table name set, retrieve table fields
		if(!empty($table) || $table == 1)
		{
			$db = \Config\Database::connect();
			if (!$db->tableExists($table))
			{
				CLI::error('"'.$table.'" table is not exists.');
				CLI::write('');
				return;
			}
			$fields = $db->getFieldData($table);

			foreach ($fields as $key => $field)
			{
				$fields[$key] = (array) $field;
				$fields[$key]['rules'] = $this->buildRules($fields[$key]);
			}
			$tablename  = $table;
		} 
		else
		{
			$fields  = array_shift($params);
			$fields  = explode(',', $fields);
			foreach ($fields as $key => $field)
			{
				$field        = trim($field);
				$fields[$key] = ['name'=>$field, 'type'=>'', 'max_length'=>'', 'default'=>'', 'primary_key'=>0];
				if($field == 'id')
				{
					$fields[$key]['type'] = 'int';
					$fields[$key]['primary_key'] = 1;
				}
				elseif($field == 'created_at' || $field == 'updated_at')
				{
					$fields[$key]['type'] = 'datetime';
				}
				$fields[$key]['rules'] = $this->buildRules($fields[$key]);
			}
		}

		// Make $allowedFields, $validationRules, $validationMessages from $fields
		$allowedFields      = [];
		$validationRules    = [];
		$validationMessages = [];
		foreach ($fields as $key => $field)
		{
			// ignore empty field
			if(empty($field['name'])) continue;

			$allowedFields[]    = "'".$field['name']."'";
			$validationRules[]  = "'".$field['name']."' => '".implode('|', $field['rules'])."'";
			$rulesMessage       = "'".$field['name']."' => [";
			foreach ($field['rules'] as $key => $rule)
			{
				$rule = preg_replace('/\[.+\]/', '', $rule);
				$rulesMessage  .="\n            '".$rule."' => NULL,";
			}
			$validationMessages[] = $rulesMessage."\n        ]";
		}
		$allowedFields      = implode(",", $allowedFields);
		$validationRules    = implode(",\n        ", $validationRules);
		$validationMessages = implode(",\n        ", $validationMessages);

		//--------------------------------------------------------------------
		// model template
		//--------------------------------------------------------------------
		$template = <<<EOT
<?php namespace App\Models\{namespace};

use App\Models\Model;
use App\Entities\{namespace}\{classname}Entity;

class {classname} extends Model
{
	protected \$table = '{tablename}';
	protected \$returnType = {classname}Entity::class;
	protected \$useTimestamps = true;
	protected \$allowedFields = [
		{allowedFields}
	];

	protected \$validationRules = [
		{validationRules}
	];

	protected \$validationMessages = [
		{validationMessages}
	];
}

EOT;
		// replace placeholders with variables
		$template = str_replace('{namespace}', $namespace, $template);
		$template = str_replace('{tablename}', $tablename, $template);
		$template = str_replace('{classname}', $classname, $template);
		$template = str_replace('{allowedFields}', $allowedFields, $template);
		$template = str_replace('{validationRules}', $validationRules, $template);
		$template = str_replace('{validationMessages}', $validationMessages, $template);
		// if directory is not existed 
		if(!file_exists(APPPATH.'Models/'.$namespace.'/'))
		{
			mkdir(APPPATH.'Models/'.$namespace.'/', 0755, true);
			CLI::write('Created: ' . CLI::color('APPPATH/Models/'.$namespace.'/', 'green'));
		}
		// create the file
		$makefile = APPPATH.'Models/'.$namespace.'/'.$classname.'.php';
		file_put_contents($makefile, $template);
		CLI::write('Created: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', $makefile), 'green'));
		CLI::write('');
	}

	public function buildRules($field)
	{
		$rules = [];
		if(empty($field['default']))
		{
			$rules['required'] = 'required';
		}
		if(isset($field['max_length']) && !empty($field['max_length']))
		{
			$rules['max_length'] = 'max_length['.$field['max_length'].']';
		}
		if($field['type'] == 'int')
		{
			$rules['type'] = 'is_natural';
		}
		elseif($field['type'] == 'datetime')
		{
			$rules['type'] = 'valid_date[Y-m-d H:i:s]';
		}
		elseif($field['type'] == '' || $field['type'] == 'varchar' || $field['type'] == 'text')
		{
			//$rules[] = 'min_length[1]';
		}
		return $rules;
	}

}
