<?php namespace App\Models;

class Model extends \CodeIgniter\Model
{

	/**
	 * If true, will set created_at, and updated_at
	 * values during insert and update routines.
	 *
	 * @var boolean
	 */
	protected $useTimestamps = true;

	/**
	 * The column used to know who insert the record
	 *
	 * @var string
	 */
	protected $createdByField = 'created_by';

	/**
	 * The column used to know who update the record
	 *
	 * @var string
	 */
	protected $updatedByField = 'updated_by';

	/**
	 * Skip the model's afterFind callback
	 *
	 * @var boolean
	 */
	protected $skipAfterFind = false;

	/**
	 * Skip the model's beforeInsert callback
	 *
	 * @var boolean
	 */
	protected $skipBeforeInsert = false;

	/**
	 * Skip the model's beforeUpdate callback
	 *
	 * @var boolean
	 */
	protected $skipBeforeUpdate = false;

	public function __construct()
	{
		parent::__construct();
		// Fill relations After Find
		$this->afterFind[] = 'afterFind';
		// Auto set created by , updated by , created at and updated at 
		$this->beforeInsert[] = 'beforeInsert';
		$this->beforeUpdate[] = 'beforeUpdate';
		// set default timezone 
		if(!session()->has('db_timezone'))
		{
			$this->db->query("SET time_zone = '".app_timezone()."'");
			session()->set('db_timezone', app_timezone());
		}
	}

	//--------------------------------------------------------------------
	// AfterFind
	//--------------------------------------------------------------------

	/**
	 * Set the value of the skipAfterFind flag.
	 *
	 * @param boolean $skip
	 *
	 * @return Model
	 */
	public function skipAfterFind(bool $skip = true)
	{
		$this->skipAfterFind = $skip;

		return $this;
	}

	public function afterFind($data)
	{
		// return $data as is 
		if (empty($data['data']) || $this->skipAfterFind) return $data;
		// if data is an object put it in array
		$entities = $data['data'];
		if (is_object($entities))
		{
			$entities = [$entities];
		}
		// find columns which need validation using model 
		$columns = [];
		foreach ($this->validationRules as $key => $rule)
		{
			if(preg_match('#in_model\[(.+)::(.+)\]#', $this->validationRules[$key], $matches))
			{
				$columns[$key] = [
					'model'  => $matches[1],
					'method' => $matches[2],
					'key'    => 'id',
					'rows'   => [],
					'ids'    => [],
				];
				if(strpos($matches[2], ',') !== false)
				{
					list($columns[$key]['method'],$columns[$key]['key']) = explode(',', $columns[$key]['method']);
				}
			}
		}
		// if $columns is empty return data as is 
		if (empty($columns)) return $data;

		// find ids for each entities  
		foreach ($entities as $key => $entity)
		{
			foreach ($columns as $column_key => $column)
			{
				// TODO: check if $entity->{$column_key} is exist
				if(isset($entity->{$column_key}))
				{
					$columns[$column_key]['ids'][$entity->{$column_key}] = $entity->{$column_key};
				}
			}
		}
		// query relatives
		foreach ($columns as $name => $column)
		{
			// if ids is empty array 
			if(empty($column['ids'])) continue;
			$modal = new $column['model']();
			// skip AfterFind
			$modal->skipAfterFind(true);
			$rows  = $modal->{$column['method']}($column['ids']);
			// rebuild array to assign by id later
			foreach ($rows as $data_key => $row)
			{
				$columns[$name]['rows'][$row->{$column['key']}] = $row;
			}
			// remove $modal and $rows
			unset($modal, $rows);
		}
		// fill each entity
		foreach ($entities as $key => $entity)
		{
			foreach ($columns as $name => $column)
			{
				if(isset($columns[$name]['rows'][$entity->{$name}]))
				{
					$entity->{$name} = $columns[$name]['rows'][$entity->{$name}];
				}
			}
		}

		$data['data'] = is_object($data['data']) ? array_shift($entities) : $entities;
		return $data;
	}

	//--------------------------------------------------------------------
	// beforeInsert
	//--------------------------------------------------------------------

	/**
	 * Set the value of the skipBeforeInsert flag.
	 *
	 * @param boolean $skip
	 *
	 * @return Model
	 */
	public function skipBeforeInsert(bool $skip = true)
	{
		$this->skipBeforeInsert = $skip;

		return $this;
	}

	public function beforeInsert($data)
	{
		// return $data as is 
		if (empty($data['data']) || $this->skipBeforeInsert) return $data;
		if (! empty($this->createdByField) && ! array_key_exists($this->createdByField, $data) && session()->has('user'))
		{
			$data['data'][$this->createdByField] = session('user')->id;
		}
		foreach ($data['data'] as $key => $value)
		{
			if(is_array($value) || is_object($value))
			{
				$data['data'][$key] = serialize($value);
			}
		}
		return $data;
	}

	//--------------------------------------------------------------------
	// beforeUpdate
	//--------------------------------------------------------------------

	/**
	 * Set the value of the skipBeforeUpdate flag.
	 *
	 * @param boolean $skip
	 *
	 * @return Model
	 */
	public function skipBeforeUpdate(bool $skip = true)
	{
		$this->skipBeforeUpdate = $skip;

		return $this;
	}

	public function beforeUpdate($data)
	{
		// return $data as is 
		if (empty($data['data']) || $this->skipBeforeUpdate) return $data;
		if (! empty($this->updatedByField) && ! array_key_exists($this->updatedByField, $data) && session()->has('user'))
		{
			$data['data'][$this->updatedByField] = session('user')->id;
		}
		foreach ($data['data'] as $key => $value)
		{
			if(is_array($value) || is_object($value))
			{
				$data['data'][$key] = serialize($value);
			}
		}
		return $data;
	}

}
